@@ -7,10 +7,11 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  u s e   v a r s   q w ( $ V E R S I O N   @ I S A ) ; 
  
  #   I n i t i a l i z e   o u r   v e r s i o n 
- $ V E R S I O N   =   ' 0 . 0 1 _ 0 1 ' ; 
+ $ V E R S I O N   =   ' 0 . 0 1 _ 0 3 ' ; 
  
  u s e   P O E ; 
  u s e   P O E : : K e r n e l ; 
+ u s e   M o d u l e : : F i n d ; 
  u s e   C a r p   q w (   c r o a k   ) ; 
  u s e   C l a s s : : A c c e s s o r : : F a s t ; 
  u s e   b a s e   ' C l a s s : : A c c e s s o r : : F a s t ' ; 
@@ -187,16 +188,16 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  u s e   c o n s t a n t   N O _ D E F A U L T _ V A L U E _ F L A G 	 = >   4 0 9 6 ; 
  u s e   c o n s t a n t   N U M _ F L A G 	 	 	 = >   3 2 7 6 8 ; 
  
- # m y   $ M Y S E R V E R _ P A C K E T _ C O U N T ; 
- # m y   $ Q U E R Y _ H A N D L E R S ; 
+ s u b   n e w   { 
+         m y   $ c l a s s   =   s h i f t ; 
+         m y   $ s e l f   =   $ c l a s s - > S U P E R : : n e w ( @ _ ) ; 
+ 
+         r e t u r n   $ s e l f ; 
+ } 
  
  s u b   _ a u t h e n t i c a t e   { 
          m y   ( $ s e l f ,   $ d a t a )   =   @ _ ; 
          
-         
-         
-         # p r i n t   "     ' " . $ d a t a . " '     \ n " ; 
-         
          m y   $ d a t a b a s e ; 
  
          m y   $ p t r   =   0 ; 
@@ -241,10 +242,20 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
                  $ s e l f - > d a t a b a s e ( $ d a t a b a s e ) ; 
                  $ s e l f - > u s e r n a m e ( $ u s e r n a m e ) ; 
  #                 $ s e l f - > p a s s w o r d ( $ d a t a b a s e ) ; 
+                 
+         } ; 
+ 
+ #         i f   ( $ @ )   { 
+ #                 d i e   $ @ ; 
+ #         } 
  
  
-                 p r i n t   $ u s e r n a m e ; 
-                 p r i n t   '   ~ ~   ' . $ d a t a b a s e . " \ n " ; 
+ 
+         i f   ( $ d a t a b a s e )   { 
+ 
+ 
+                 p r i n t   $ s e l f - > u s e r n a m e ; 
+                 p r i n t   '   ~ ~   ' . $ s e l f - > d a t a b a s e . " \ n " ; 
  
                  m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
  
@@ -262,17 +273,11 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
                          } 
  
                          u n l e s s   ( $ @ )   { 
-                                 $ s e l f - > d a t a b a s e ( $ d a t a ) ; 
+                                 $ s e l f - > d a t a b a s e ( $ d a t a b a s e ) ; 
                          } 
                  } 
-                 
-         } ; 
  
-         i f   ( $ @ )   { 
-                 d i e   $ @ ; 
-         } 
  
-         i f   ( $ d a t a b a s e )   { 
                  $ s e l f - > a u t h e n t i c a t e d ( 1 ) ; 
                  $ s e l f - > s e n d _ o k ; 
          } 
@@ -281,93 +286,38 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
          } 
  } 
  
- 
- 
  s u b   s e n d _ e r r o r   { 
- # # #   S e n d i n g   E r r o r   . . . 
- 	 m y   ( $ s e l f ,   $ m e s s a g e ,   $ e r r n o ,   $ s q l s t a t e )   =   @ _ ; 
- # # # #   $ e r r n o 
- # # # #   $ s q l s t a t e 
- # # # #   $ m e s s a g e 
- 	 #   h t t p : / / d e v . m y s q l . c o m / d o c / i n t e r n a l s / e n / e r r o r - p a c k e t . h t m l 
+         m y   ( $ s e l f ,   $ m e s s a g e ,   $ e r r n o ,   $ s q l s t a t e )   =   @ _ ; 
  
- 	 $ m e s s a g e   =   ' U n k n o w n   M y S Q L   e r r o r '   i f   n o t   d e f i n e d   $ m e s s a g e ; 
- 	 $ e r r n o   =   2 0 0 0   i f   n o t   d e f i n e d   $ e r r n o ; 
- 	 $ s q l s t a t e   =   ' H Y 0 0 0 '   i f   n o t   d e f i n e d   $ s q l s t a t e ; 
- 
- 	 m y   $ p a y l o a d   =   c h r ( 0 x f f ) ; 	 	 #   f i e l d _ c o u n t ,   a l w a y s   =   0 x f f 
- 	 $ p a y l o a d   . =   p a c k ( ' v ' ,   $ e r r n o ) ; 
- 	 $ p a y l o a d   . =   ' # ' ; 	 	 	 	 #   s q l s t a t e   m a r k e r ,   a l w a y s   # 
- 	 $ p a y l o a d   . =   $ s q l s t a t e ; 
- 	 $ p a y l o a d   . =   $ m e s s a g e . " \ 0 " ; 
+         $ m e s s a g e   =   ' U n k n o w n   M y S Q L   e r r o r '   i f   n o t   d e f i n e d   $ m e s s a g e ; 
+         $ e r r n o   =   2 0 0 0   i f   n o t   d e f i n e d   $ e r r n o ; 
+         $ s q l s t a t e   =   ' H Y 0 0 0 '   i f   n o t   d e f i n e d   $ s q l s t a t e ; 
  
+         m y   $ p a y l o a d   =   c h r ( 0 x f f ) ; 	 	 #   f i e l d _ c o u n t ,   a l w a y s   =   0 x f f 
+         $ p a y l o a d   . =   p a c k ( ' v ' ,   $ e r r n o ) ; 
+         $ p a y l o a d   . =   ' # ' ; 	 	 	 	 #   s q l s t a t e   m a r k e r ,   a l w a y s   # 
+         $ p a y l o a d   . =   $ s q l s t a t e ; 
+         $ p a y l o a d   . =   $ m e s s a g e . " \ 0 " ; 
  
          $ s e l f - > w r i t e (   $ p a y l o a d ,   1 ) ; 
- 
  } 
  
- 
-   
- # s u b   r e s o l v e _ q u e r y   { 
- #         m y   ( $ s e l f ,   $ q u e r y )   =   @ _ ; 
- # 
- #         $ s e l f - > S U P E R : : r e s o l v e _ q u e r y ( @ _ ) ; 
- # 
- #         m y   $ s y s _ q u e r y   =   $ s e l f - > r e s o l v e _ s y s _ q u e r y ( $ q u e r y ) ; 
- # 
- #         p r i n t   "   r e f   " . r e f ( $ s e l f ) . "   \ n " ; 
- # 
- # 
- #         p r i n t   D u m p e r ( \ @ I S A ) ; 
- #         r e t u r n   $ s y s _ q u e r y   i f   d e f i n e d   $ s y s _ q u e r y ; 
- # 
- #         p r i n t   " p a r e n t   r e s o l v e   q u e r y   \ n " ; 
- # 
- # 
- # #         i f   ( $ q u e r y   e q   ' s e l e c t   @ @ v e r s i o n _ c o m m e n t   l i m i t   1 ' )   { 
- # #                 r e t u r n   ' t e s t ' ; 
- # #         } 
- #         r e t u r n   ' t e s t _ t a b l e s ' ; 
- # } 
- 
- 
- s u b   r e s o l v e _ s y s _ q u e r y   { 
-         m y   ( $ s e l f ,   $ q u e r y )   =   @ _ ; 
- 
-         i f   ( $ q u e r y   e q   ' s e l e c t   @ @ v e r s i o n _ c o m m e n t   l i m i t   1 ' )   { 
-                 r e t u r n   ' t e s t ' ; 
-         } 
-         e l s i f   ( $ q u e r y   = ~   / a r r a y / )   { 
-                 r e t u r n   ' t e s t _ m y a r r a y ' ; 
-         } 
-         e l s i f   ( $ q u e r y   = ~   / d a t a b a s e s / )   { 
-                 r e t u r n   ' t e s t _ d b ' ; 
-         } 
-         e l s i f   ( $ q u e r y   e q   ' s h o w   t a b l e s ' )   { 
-                 r e t u r n   ' s h o w _ t a b l e s ' ; 
-         } 
-         e l s i f   ( $ q u e r y   = ~   / ^ s e l e c t   d a t a b a s e / i )   { 
-                 r e t u r n   ' s e l e c t _ d b ' ; 
-         } 
-         e l s i f   ( $ q u e r y   = ~   / ^ u s e /   o r   $ q u e r y   = ~   / ^ t e s t _ c h a n g e _ d b / i   o r   $ q u e r y   = ~   / ^ s e l e c t   d a t a b a s e / i )   { 
-                 r e t u r n   ' c h a n g e _ d b ' ; 
-         } 
- 
-         r e t u r n   u n d e f ; 
- } 
- 
- 
  s u b   i s a   { 
          m y   ( $ s e l f ,   $ m o d u l e )   =   @ _ ; 
  
  
          i f   ( $ m o d u l e )   { 
-         p r i n t   " i s a   - >   $ m o d u l e   \ n " ; 
                  u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
                          r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
                  } 
+ 
+                 p r i n t   D u m p e r ( \ @ I S A ) ; 
+ 
+                 p o p   @ I S A   i f   $ I S A [ $ # I S A ]   n e   ' C l a s s : : A c c e s s o r : : F a s t ' ; 
                  p u s h   @ I S A ,   $ m o d u l e ; 
  
+                 p r i n t   D u m p e r ( \ @ I S A ) ; 
+ 
                  #   r e g i s t e r   s y s t e m   s t a t e s         
                  m y   @ m e t h o d s   =   C l a s s : : I n s p e c t o r - > m e t h o d s ( 
                          $ m o d u l e , 
@@ -385,9 +335,6 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  
          } 
  
- u s e   D a t a : : D u m p e r ; 
- p r i n t   D u m p e r ( \ @ I S A ) ; 
- 
          r e t u r n   @ I S A ; 
  } 
  
@@ -424,9 +371,8 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
                                  p r i n t   $ @ . " \ n " ; 
                                  $ e v e n t   =   $ s e l f - > r e s o l v e _ s y s _ q u e r y ( $ d a t a ) ; 
                          } 
-                     
-                         i f   ( $ s e l f - > w a n t _ d b _ n a m e )   { 
-                 #                         p r i n t   " q u e r y   i s   D B   n a m e   ! !   \ n " ; 
+ 
+                         i f   ( $ s e l f - > w a n t _ d b _ n a m e   & &   C l a s s : : I n s p e c t o r - > i n s t a l l e d ( $ s e l f - > s e r v e r _ c l a s s . " : : " . $ d a t a ) )   { 
                                  $ e v e n t   =   ' s e l e c t _ d b ' ; 
                          } 
                  } ; 
@@ -440,12 +386,26 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
        } 
  } 
  
+ s u b   r e s o l v e _ s y s _ q u e r y   { 
+         m y   ( $ s e l f ,   $ q u e r y )   =   @ _ ; 
  
- s u b   n e w   { 
-         m y   $ c l a s s   =   s h i f t ; 
-         m y   $ s e l f   =   $ c l a s s - > S U P E R : : n e w ( @ _ ) ; 
+         i f   ( $ q u e r y   e q   ' s e l e c t   @ @ v e r s i o n _ c o m m e n t   l i m i t   1 ' )   { 
+                 r e t u r n   ' s e l e c t _ v e r s i o n ' ; 
+         } 
+         e l s i f   ( $ q u e r y   = ~   / a r r a y / )   { 
+                 r e t u r n   ' t e s t _ m y a r r a y ' ; 
+         } 
+         e l s i f   ( $ q u e r y   e q   ' s h o w   d a t a b a s e s ' )   { 
+                 r e t u r n   ' s h o w _ d a t a b a s e s ' ; 
+         } 
+         e l s i f   ( $ q u e r y   e q   ' s h o w   t a b l e s ' )   { 
+                 r e t u r n   ' s h o w _ t a b l e s ' ; 
+         } 
+         e l s i f   ( $ q u e r y   = ~   / ^ u s e /   o r   $ q u e r y   = ~   / ^ s e l e c t   d a t a b a s e / i )   { 
+                 r e t u r n   ' s e l e c t _ d b ' ; 
+         } 
  
-         r e t u r n   $ s e l f ; 
+         r e t u r n   u n d e f ; 
  } 
  
  
@@ -635,10 +595,10 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
          m y   ( $ s e l f ,   $ r o w s )   =   @ _ ; 
          $ s e l f - > s e n d _ e o f   i f   n o t   d e f i n e d   $ r o w s ; 
          
- 	 f o r e a c h   m y   $ r o w   ( @ { $ r o w s } )   { 
+ 	 f o r e a c h   m y   $ r o w   ( @ $ r o w s )   { 
                  
                  m y   $ s m a l l _ d a t a ; 
-                 f o r e a c h   ( @ { $ r o w } )   { 
+                 f o r e a c h   ( @ $ r o w )   { 
                          i f   ( n o t   d e f i n e d   $ _ )   { 
                                  $ s m a l l _ d a t a   . =   c h r ( 2 5 1 ) ; 
                          } 
@@ -666,17 +626,11 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
          $ s e l f - > _ s e n d _ r o w s ( $ d a t a ) ; 
  } 
  
- s u b   t e s t   { 
+ s u b   s e l e c t _ v e r s i o n   { 
          m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
          m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
- 
-         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
-       
-         p r i n t   " v e r s i o n _ c o m m e n t   " . $ s e l f . "   ! !   \ n " ;   
          
-         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' f i e l d '   ) ] ,   [ [ ' e c h o . p l   ~   ' . $ s e s s i o n - > I D ] ] ) ; 
- 
-         p r i n t   " d a   s h i t   w a s   s e n t   . . .   \ n " ; 
+         $ s e l f - > s e n d _ r e s u l t s ( [ ' v e r s i o n ' ] ,   [ [   ' o n   p e r l   ' . $ ]   ] ] ) ; 
  } 
  
  s u b   t e s t _ m y a r r a y   { 
@@ -701,7 +655,7 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  
  s u b   t e s t _ t a b l e s   { 
          m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
-         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
+       m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
  
          p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
        
@@ -719,68 +673,21 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  s u b   s h o w _ t a b l e s   { 
          m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
          m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
- 
-         $ s e l f - > s e n d _ r e s u l t s ( [ ' T a b l e s _ i n _ t e s t d b ' ] , 
-                 [ ] ) ;       	 
- 
+         #   s o   m y s q l   c l i e n t   d o n ' t   c o m p l a i n   a b o u t   s t u f f   h e   
+         #   d o e s n t   k n o w   f r o m   e a c h   t a b l e   . . 
+         $ s e l f - > s e n d _ r e s u l t s ( [ ' T a b l e s _ i n _ ' . $ s e l f - > d a t a b a s e ] , [ ] ) ; 
  } 
  
- # s u b   c h a n g e _ d b   { 
- #         m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
- #         m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
- # 
- #         #   
- #         #   m o d u l e   l o a d   a n d   s h i t   . . . 
- #         # 
- #         m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
- # #         p r i n t   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
- # 
- #         u n l e s s   ( C l a s s : : I n s p e c t o r - > l o a d e d ( $ m o d u l e ) )   { 
- #                 r e q u i r e   C l a s s : : I n s p e c t o r - > f i l e n a m e ( $ m o d u l e ) ; 
- #         } 
- # 
- #         i f   ( m y   $ c o d e   =   $ m o d u l e - > c a n ( ' c h a n g e _ d b ' ) )   { 
- #                 $ c o d e - > ( $ m o d u l e ,   $ s e l f ,   $ d a t a ) ;   
- #         } 
- # 
- # # 
- # #                 P O E : : K e r n e l - > p o s t ( 
- # #                         $ s e l f - > s e s s i o n _ i d ,   
- # #                         ' a _ d u m m y _ q u e r y ' ,   
- # #                         $ d a t a 
- # #                 ) ; 
- # 
- #         
- # 
- # #         e v a l   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
- # 
- #         p r i n t   ' c h a n g e   d b   ' . $ d a t a . "   o k   ! ! \ n " ; 
- # 
- # } 
- 
  s u b   s e l e c t _ d b   { 
          m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
          m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
  
-         p r i n t   ' t e s t   ~   s e s s i o n   I D   =   ' . $ s e s s i o n - > I D . " \ n " ; 
-       
- 
-         #   
          #   m o d u l e   l o a d   a n d   s h i t   . . . 
-         # 
          m y   $ m o d u l e   =   $ s e l f - > s e r v e r _ c l a s s ; 
- #         p r i n t   ( $ p a c k a g e . " - > c h a n g e _ d b ( " . $ d a t a . " ) " ) ; 
- 
-       	     p r i n t   " s e l e c t   D B   $ d a t a   \ n " ; 
        	 
          i f   ( $ d a t a   = ~   / \ ( \ ) / )   { 
                  $ s e l f - > w a n t _ d b _ n a m e ( 1 ) ; 
-                 p r i n t   " I   w a n t   t h e   n a m e   o f   t h e   d b   \ n " ; 
-                 $ d a t a   =   ' t e s t d b ' ; 
- 
- #                 $ s e l f - > s e n d _ r e s u l t s ( [ ' D a t a b a s e ' ] ,   [ [ ' t e s t d b ' ] ] ) ;   
                  $ s e l f - > s e n d _ r e s u l t s ( [ ' d a t a b a s e ( ) ' ] ,   [ [ $ s e l f - > d a t a b a s e ] ] ) ;   
- 
          } 
          e l s e   { 
  
@@ -797,41 +704,31 @@ p a c k a g e   P O E : : C o m p o n e n t : : D B I x : : M y S e r v e r : : 
  
                          u n l e s s   ( $ @ )   { 
                                  $ s e l f - > d a t a b a s e ( $ d a t a ) ; 
+ #                                 $ s e l f - > s e n d _ r e s u l t s ( [ ' d a t a b a s e ( ) ' ] ,   [ [ $ d a t a ] ] ) ;   	 
+                                 $ s e l f - > s e n d _ o k ;                               
+                         } 
+                         e l s e   { 
+                                 $ s e l f - > s e n d _ e r r o r ( $ @ ) ;               
                          } 
                  } 
- 
-                 
- 
-     p r i n t   " d a   " . $ d a t a . "   D B   w a s   s e n t   . . .   \ n " ; 
-                 $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] ,   [ [ $ d a t a ] ] ) ;   	 
- 
          } 
- 
- 
- #         $ d a t a   | | =   ' t e s t d b ' ; 
- 
- 
- #             $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , 
- #                 [ [ ' t e s t d b ' ] , [ ' m y d b ' ] , [ ' D a t a b a s e T e s t ' ] , [ ' t e s t d b ' ] ] ) ;   	 
- 
-     
- 
- #         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , [ [ ' t e s t d b ' ] ] ) ;   	 
- 
- 
-         
- 
  } 
- s u b   t e s t _ d b   { 
+ 
+ 
+ s u b   s h o w _ d a t a b a s e s   { 
          m y   (   $ k e r n e l ,   $ s e s s i o n ,   $ h e a p ,   $ s e l f   )   =   @ _ [   K E R N E L ,   S E S S I O N ,   H E A P ,   O B J E C T ] ; 
          m y   $ d a t a   =   $ _ [ A R G 0 ] ; 
+         m y   @ r e s u l t s ; 
  
-       
-         p r i n t   " s h o w   d a t a b a s e s   ! !   \ n " ; 
-       	 
-         $ s e l f - > s e n d _ r e s u l t s ( [ n e w _ d e f i n i t i o n (   n a m e   = >   ' D a t a b a s e '   ) ] , 
-                 [ [ ' m y d b ' ] , [ ' D a t a b a s e T e s t ' ] , [ ' t e s t d b ' ] ] ) ;   	 
+         m y   $ s e r v e r _ c l a s s   =   $ s e l f - > s e r v e r _ c l a s s ; 
+         m y   @ f o u n d   =   u s e a l l   $ s e l f - > s e r v e r _ c l a s s ; 
  
+         f o r e a c h   m y   $ d a t a b a s e _ c l a s s   ( @ f o u n d )   { 
+                 $ d a t a b a s e _ c l a s s   = ~   s / $ s e r v e r _ c l a s s \ : \ : / / g ; 
+                 p u s h   @ r e s u l t s ,   [ $ d a t a b a s e _ c l a s s ] ; 
+         }       
+ 
+         $ s e l f - > s e n d _ r e s u l t s ( [ ' D a t a b a s e ' ] ,   \ @ r e s u l t s ) ;   	 
  } 
  
  s u b   n e w _ d e f i n i t i o n   { 
